unit ExtendedGLBox;

{$mode ObjFPC}{$H+}

interface

// -----------------------------------------------------------------------------
//                                2025-02-26
//
//                           OpenGL Test Object
//
//  ----------------------------------------------------------------------------


uses
  Classes, Forms, Controls, ExtCtrls, Dialogs, SysUtils,
  OpenGLContext, glmtext, gl_core_utils, gl_core_matrix, glcorearb;


type
  { TFrameOpenGL }
  TOpenGLBox = class(TOpenGLControl)
    protected
      procedure GLboxMouseDown(Sender: TObject; {%H-}Button: TMouseButton;
        {%H-}Shift: TShiftState; X, Y: Integer);
      procedure GLboxMouseMove(Sender: TObject; {%H-}Shift: TShiftState; X, Y: Integer
        );
      procedure GLboxMouseUp(Sender: TObject; {%H-}Button: TMouseButton;
        {%H-}Shift: TShiftState; {%H-}X, {%H-}Y: Integer);
      procedure GLboxMouseWheel(Sender: TObject; {%H-}Shift: TShiftState;
        WheelDelta: Integer; {%H-}MousePos: TPoint; var {%H-}Handled: Boolean);
    private
      Timer1   : TTimer;
      gGLText1 : TGLText;
      gGLText2 : TGLText;
      gGLText3 : TGLText;
      basenm   : string;
      procedure GLboxPaint(Sender: TObject);
      procedure InitGL;
      procedure InitTGLText( var GLText: TGLText);
      procedure Timer1Timer(Sender: TObject);
      procedure UpdateGLText;
      procedure UpdateMovableGLText;
    public
      constructor Create(TheOwner: TComponent); override;
      procedure AfterConstruction; override;
  end;


implementation

uses
  Mainform;


var
  gPositionX : single = 50;
  gPositionY : single = 5;
  gZoom      : single = 0.6;
  gMouseY    : integer = -1;
  gMouseX    : integer = -1;


constructor TOpenGLBox.Create(TheOwner: TComponent);
begin
  inherited;
end;

procedure TOpenGLBox.AfterConstruction;
begin
  inherited;

  Self.Parent:= Mainform.GLForm1;
  Self.Left:= 20;
  Self.Top:= 20;
  Self.Width:= Mainform.GLForm1.Width -40;
  Self.Height:= Mainform.GLForm1.Height -40;

  //OSX has legacy and core modes:
  //    NSOpenGLProfileLegacy provides support for OpenGL 2.1/GLSL1.2 and earlier
  //    NSOpenGLProfileVersion3_2Core provides support for AT LEAST OpenGL 3.2/GLSL3.2 CORE
  //NOTE: CORE support removes deprecated LEGACY features
  //  In other words, Core OpenGL3.2 is NOT a superset of OpenGL2.1
  //  Functions like gl_FragColor, glTranslate etc. do not exist in CORE OpenGL
  //  Therefore, CORE is similar to embedded OpenGL, and old tutorials will not work
  {$IFDEF LCLCarbon}
   Error: Carbon only supports Legacy OpenGL. Solution: compile to the Cocoa widgetset (Project/ProjectOptions/Additions&Overrides)
  {$ENDIF}
  InitGL;
  {$IFDEF RETINA}
  SetRetina;
  {$ENDIF}
  basenm:= extractfilepath( paramstr(0));
  {$IFDEF DARWIN}
  basenm := extractfilepath(ExcludeTrailingPathDelimiter(basenm))+'Resources/';
  {$ENDIF}

  // load 'Roboto.png' and 'Roboto.json'
  Self.InitTGLText( gGLText1);
  Self.InitTGLText( gGLText2);
  Self.InitTGLText( gGLText3);


  // manually assign these functions
  OnPaint      := @GLboxPaint;
  OnMouseWheel := @GLboxMouseWheel;
  OnMouseMove  := @GLboxMouseMove;
  OnMouseDown  := @GLboxMouseDown;
  OnMouseUp    := @GLboxMouseUp;

  if GLErrorStr <> '' then
    begin
      ShowMessage( GLErrorStr);
      GLErrorStr := '';
    end;

  Timer1:= TTimer.Create( Self);
  Timer1.OnTimer:= @Timer1Timer;
  Timer1.Enabled:= TRUE;

  UpdateMovableGLText;
end;

procedure TOpenGLBox.InitGL;//( var GLcontrol: TOpenGLControl);
begin
  Self.MakeCurrent();

  if not Load_GL_version_3_3_CORE() then
    begin
     Self.ReleaseContext;
     {$IFNDEF Windows} writeln('Unable to load OpenGL');{$ENDIF}
     ShowMessage('Unable to load OpenGL 3.3');
     Halt();
    end;

  Self.ReleaseContext;
  if GLErrorStr <> '' then
    begin
      ShowMessage(GLErrorStr);
      GLErrorStr := '';
    end;
end;

procedure TOpenGLBox.InitTGLText(var GLText: TGLText);
var
  success : boolean;
  fnm     : string;
begin
  TRY
    fnm := basenm + 'Roboto.png';
    GLText := GLMText.TGLText.Create( fnm, success, Self);
  EXCEPT
    ShowMessage( 'Exception in GLMText.TGLText.Create()');
    success:= FALSE;
  end;

  if not success
    then ShowMessage( 'Error: unable to load .png or .json ' +#13#13
                      +fnm);
end;

procedure TOpenGLBox.GLboxPaint(Sender: TObject);
begin
  nglMatrixMode( nGL_PROJECTION);
  nglLoadIdentity();
  {$IFDEF RETINA}
  nglOrtho( 0, Round(GLBox.Width * LBackingScaleFactor(GLBox.Handle)), 0, Round(GLBox.Height * LBackingScaleFactor(GLBox.Handle)), 0.1, 40);
  {$ELSE}
  nglOrtho( 0, Self.ClientWidth, 0, Self.ClientHeight, 0.1, 40);
  {$ENDIF}
  glClearColor( 0.36, 0.36, 0.78, 1.0); //Set background
  glDisable( GL_DEPTH_TEST);
  glClear( GL_COLOR_BUFFER_BIT);
  glEnable( GL_BLEND);
  glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  gGLText1.DrawText;
  gGLText2.DrawText;
  gGLText3.DrawText;

  Self.SwapBuffers;
end;

procedure TOpenGLBox.UpdateGLText;
begin
  // Date
  gGLText1.ClearText;
  gGLText1.TextColor( 160, 160, 160);
  gGLText1.TextOutHighDPI( Scale96ToForm( 110),
                           Scale96ToForm( 144),
                           0.8,
                           Screen.PixelsPerInch,
                           DateToStr( Now));
  //gGLText2.TextOut(6,36,1, 'Pack my box with five dozen liquor jugs.');
  //gGLText2.TextOut(6,16,0.5, 'Sphinx of black quartz, judge my vow.');

  // Time
  gGLText2.ClearText;
  gGLText2.TextColor( 255, 255, 255);
  gGLText2.TextOutHighDPI( 32,
                           Self.Height * 2 div 9,
                           1.8,
                           Screen.PixelsPerInch,
                           TimeToStr( Now));

  //gGLText2.TextColor( 160, 160, 160);
  //gGLText2.TextOut( Scale96ToForm( 32),
  //                  Self.Height * 1 div 9 +15,
  //                  (Scale96ToForm( 23) / 70),
   //                 'abcde fghij klmno pqrst uvwxyz äöüß @ <> °C');

  Self.Invalidate;
end;

procedure TOpenGLBox.UpdateMovableGLText;
begin
  // You can move or zoom this text with Mouse Wheel:
  gGLText3.ClearText;
  gGLText3.TextColor( 220, 220, 220);
  gGLText3.TextOutHighDPI( gPositionX,
                           gPositionY,
                           gZoom,
                           Screen.PixelsPerInch,
                           DateToStr( Now));

  Self.Invalidate;
end;

procedure TOpenGLBox.Timer1Timer(Sender: TObject);
begin
  UpdateGLText;
end;


// --- GLboxMouse actions ------------------------------------------------------
procedure TOpenGLBox.GLboxMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  {$IFDEF RETINA}
  if (ssShift in Shift) then begin
     gIsRetina := not gIsRetina;
     SetRetina;
     gPositionX := 0;
     gPositionY := 0;
     UpdateText;
  end;
 {$ENDIF}
  gMouseY:= Y;
  gMouseX:= X;
end;

procedure TOpenGLBox.GLboxMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  gMouseY:= -1; //released
end;

procedure TOpenGLBox.GLboxMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if gMouseY < 0 then exit; //mouse is not down
  if (X <> gMouseX) or (Y <> gMouseY) then
    begin
      gPositionX := gPositionX + (X - gMouseX);
      gPositionY := gPositionY + (gMouseY - Y);
    end;

  gMouseY := Y;
  gMouseX := X;
  UpdateMovableGLText;
end;

procedure TOpenGLBox.GLboxMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  if WheelDelta = 0 then exit;

  if WheelDelta > 0
    then gZoom := gZoom *1.2
    else gZoom := gZoom /1.2;

  if gZoom < 0.1 then gZoom := 0.1;
  if gZoom > 10 then gZoom := 10;

  UpdateMovableGLText;
end;


end.

